/*
  bootscrn.c - Install my customised vidstub.sys for non-BootSkin users.

  Jason Hood, 7 & 8 August, 2007.

  1 September, 2007:
  * use "Video Init" group instead of "Boot Bus Extender", but Vista might
    still occasionally miss it.
  * tweaked the output.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define STREQ( str1, str2 ) (strcmp( str1, str2 ) == 0)


char vidstub[MAX_PATH];
char oldstub[MAX_PATH];

// Create the vidstub.sys and vidstub.old file names in the system directory.
void get_vid( void )
{
  int len;

  char* root = getenv( "SystemRoot" );
  if (root == NULL)
  {
    puts( "Where's the \"SystemRoot\" environment variable?" );
    exit( 1 );
  }

  len = sprintf( vidstub, "%s\\system32\\drivers\\vidstub.sys", root );
  memcpy( oldstub, vidstub, len - 3 );
  strcpy( oldstub + len - 3, "old" );
}


BOOL install_vid( void )
{
  HANDLE scm, svc;
  BOOL	 rc = TRUE;

  if (!MoveFile( vidstub, oldstub ))
  {
    scm = OpenSCManager( NULL, NULL, SC_MANAGER_CREATE_SERVICE );
    if (scm == NULL)
    {
      printf( "Unable to access the Services database (error %lu; must run as admin).\n", GetLastError() );
      return FALSE;
    }
    svc = CreateService( scm, "BootScreen", "BootScreen",
			 SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER,
			 SERVICE_BOOT_START, SERVICE_ERROR_NORMAL,
			 vidstub, "Video Init", NULL, NULL, NULL, NULL );
    if (svc == NULL)
    {
      printf( "Unable to install the service (error %lu).", GetLastError() );
      rc = FALSE;
    }
    else
      CloseServiceHandle( svc );
    CloseServiceHandle( scm );
  }

  return rc;
}


BOOL uninstall_vid( void )
{
  HANDLE scm, svc;
  BOOL	 rc = TRUE;

  // If it can't delete the vidstub, let's assume it's not actually installed.
  if (!DeleteFile( vidstub ))
    return TRUE;

  // If the old stub doesn't exist, remove the service.
  if (!MoveFile( oldstub, vidstub ))
  {
    scm = OpenSCManager( NULL, NULL, SC_MANAGER_ALL_ACCESS );
    if (scm == NULL)
    {
      printf( "Unable to access the Services database (error %lu; must run as admin).\n", GetLastError() );
      return FALSE;
    }
    svc = OpenService( scm, "BootScreen", DELETE );
    if (svc == NULL)
    {
      printf( "Unable to delete the service (error %lu).", GetLastError() );
      rc = FALSE;
    }
    else
    {
      DeleteService( svc );
      CloseServiceHandle( svc );
    }
    CloseServiceHandle( scm );
  }

  return rc;
}


int main( int argc, char* argv[] )
{
  BOOL rc;

  if (argc == 1 || STREQ( argv[1], "/?" ) || STREQ( argv[1], "-?" ) ||
		   STREQ( argv[1], "/h" ) || STREQ( argv[1], "-h" ) ||
		   STREQ( argv[1], "--help" ))
  {
    puts( "Install a customised boot screen for Windows XP/Vista.\n"
	  "\n"
	  "bootscrn -i\tInstall.\n"
	  "bootscrn file\tUse FILE as the boot screen.\n"
	  "bootscrn -u\tUninstall.\n"
	  "\n"
	  "Jason Hood <jadoxa@yahoo.com.au>, 1 September, 2007." );
    return 0;
  }

  get_vid();

  if (STREQ( argv[1], "-i" ))
    rc = install_vid();
  else if (STREQ( argv[1], "-u" ))
    rc = uninstall_vid();
  else
  {
    rc = CopyFile( argv[1], vidstub, FALSE );
    if (!rc)
    {
      DWORD err = GetLastError();
      if (err == ERROR_FILE_NOT_FOUND || err == ERROR_PATH_NOT_FOUND)
	printf( "\"%s\" does not seem to exist.\n", argv[1] );
      else if (err == ERROR_ACCESS_DENIED)
	printf( "Unable to copy \"%s\" to \"%s\" (try running as admin).\n", argv[1], vidstub );
      else
	printf( "Unable to copy \"%s\" to \"%s\" (error %lu).\n", argv[1], vidstub, err );
    }
  }

  return !rc;
}
